package sun.security.x509;

import java.util.Arrays;
import java.security.Key;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import sun.misc.HexDumpEncoder;
import java.security.Provider;
import java.security.spec.InvalidKeySpecException;
import java.security.NoSuchAlgorithmException;
import java.security.Security;
import java.security.KeyFactory;
import java.security.spec.X509EncodedKeySpec;
import sun.security.util.DerOutputStream;
import java.io.IOException;
import sun.security.util.DerValue;
import java.security.InvalidKeyException;
import sun.security.util.BitArray;
import java.security.PublicKey;

public class X509Key implements PublicKey
{
    private static final long serialVersionUID = -5359250853002055002L;
    protected AlgorithmId algid;
    @Deprecated
    protected byte[] key;
    @Deprecated
    private int unusedBits;
    private BitArray bitStringKey;
    protected byte[] encodedKey;
    
    public X509Key() {
        this.key = null;
        this.unusedBits = 0;
        this.bitStringKey = null;
    }
    
    private X509Key(AlgorithmId algid, BitArray key) throws InvalidKeyException {
        this.key = null;
        this.unusedBits = 0;
        this.bitStringKey = null;
        this.algid = algid;
        this.setKey(key);
        this.encode();
    }
    
    protected void setKey(BitArray bitArray) {
        this.bitStringKey = (BitArray)bitArray.clone();
        this.key = bitArray.toByteArray();
        int n = bitArray.length() % 8;
        this.unusedBits = ((n == 0) ? 0 : (8 - n));
    }
    
    protected BitArray getKey() {
        this.bitStringKey = new BitArray(this.key.length * 8 - this.unusedBits, this.key);
        return (BitArray)this.bitStringKey.clone();
    }
    
    public static PublicKey parse(DerValue derValue) throws IOException {
        if (derValue.tag != 48) {
            throw new IOException("corrupt subject key");
        }
        AlgorithmId parse = AlgorithmId.parse(derValue.data.getDerValue());
        PublicKey buildX509Key;
        try {
            buildX509Key = buildX509Key(parse, derValue.data.getUnalignedBitString());
        }
        catch (InvalidKeyException cause) {
            throw new IOException("subject key, " + cause.getMessage(), cause);
        }
        if (derValue.data.available() != 0) {
            throw new IOException("excess subject key");
        }
        return buildX509Key;
    }
    
    protected void parseKeyBits() throws IOException, InvalidKeyException {
        this.encode();
    }
    
    static PublicKey buildX509Key(AlgorithmId algid, BitArray key) throws IOException, InvalidKeyException {
        DerOutputStream derOutputStream = new DerOutputStream();
        encode(derOutputStream, algid, key);
        X509EncodedKeySpec keySpec = new X509EncodedKeySpec(derOutputStream.toByteArray());
        try {
            return KeyFactory.getInstance(algid.getName()).generatePublic(keySpec);
        }
        catch (NoSuchAlgorithmException ex) {
            String property = "";
            try {
                Provider provider = Security.getProvider("SUN");
                if (provider == null) {
                    throw new InstantiationException();
                }
                property = provider.getProperty("PublicKey.X.509." + algid.getName());
                if (property == null) {
                    throw new InstantiationException();
                }
                Class<Object> clazz = null;
                try {
                    clazz = Class.forName(property);
                }
                catch (ClassNotFoundException ex2) {
                    ClassLoader systemClassLoader = ClassLoader.getSystemClassLoader();
                    if (systemClassLoader != null) {
                        clazz = systemClassLoader.loadClass(property);
                    }
                }
                Object instance = null;
                if (clazz != null) {
                    instance = clazz.newInstance();
                }
                if ((instance instanceof X509Key)) {
                    X509Key x509Key = (X509Key)instance;
                    x509Key.algid = algid;
                    x509Key.setKey(key);
                    x509Key.parseKeyBits();
                    return x509Key;
                }
            }
            catch (ClassNotFoundException ex3) {}
            catch (InstantiationException ex4) {}
            catch (IllegalAccessException ex5) {
                throw new IOException(property + " [internal error]");
            }
            return new X509Key(algid, key);
        }
        catch (InvalidKeySpecException ex6) {}
    }
    
    @Override
    public String getAlgorithm() {
        return this.algid.getName();
    }
    
    public AlgorithmId getAlgorithmId() {
        return this.algid;
    }
    
    public final void encode(DerOutputStream derOutputStream) throws IOException {
        encode(derOutputStream, this.algid, this.getKey());
    }
    
    @Override
    public byte[] getEncoded() {
        try {
            return (byte[])this.getEncodedInternal().clone();
        }
        catch (InvalidKeyException ex) {
            return null;
        }
    }
    
    public byte[] getEncodedInternal() throws InvalidKeyException {
        byte[] encodedKey = this.encodedKey;
        if (encodedKey == null) {
            try {
                DerOutputStream derOutputStream = new DerOutputStream();
                this.encode(derOutputStream);
                encodedKey = derOutputStream.toByteArray();
            }
            catch (IOException ex) {
                throw new InvalidKeyException("IOException : " + ex.getMessage());
            }
            this.encodedKey = encodedKey;
            return encodedKey;
        }
        return encodedKey;
    }
    
    @Override
    public String getFormat() {
        return "X.509";
    }
    
    public byte[] encode() throws InvalidKeyException {
        return (byte[])this.getEncodedInternal().clone();
    }
    
    @Override
    public String toString() {
        return "algorithm = " + this.algid.toString() + ", unparsed keybits = \n" + new HexDumpEncoder().encodeBuffer(this.key);
    }
    
    public void decode(InputStream in) throws InvalidKeyException {
        try {
            DerValue derValue = new DerValue(in);
            if (derValue.tag != 48) {
                throw new InvalidKeyException("invalid key format");
            }
            this.algid = AlgorithmId.parse(derValue.data.getDerValue());
            this.setKey(derValue.data.getUnalignedBitString());
            this.parseKeyBits();
            if (derValue.data.available() != 0) {
                throw new InvalidKeyException("excess key data");
            }
        }
        catch (IOException ex) {
            throw new InvalidKeyException("IOException: " + ex.getMessage());
        }
    }
    
    public void decode(byte[] buf) throws InvalidKeyException {
        this.decode(new ByteArrayInputStream(buf));
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.write(this.getEncoded());
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException {
        try {
            this.decode(objectInputStream);
        }
        catch (InvalidKeyException ex) {
            ex.printStackTrace();
            throw new IOException("deserialized key is invalid: " + ex.getMessage());
        }
    }
    
    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof Key)) {
            return false;
        }
        try {
            byte[] encodedInternal = this.getEncodedInternal();
            byte[] a2;
            if ((o instanceof X509Key)) {
                a2 = ((X509Key)o).getEncodedInternal();
            }
            else {
                a2 = ((Key)o).getEncoded();
            }
            return Arrays.equals(encodedInternal, a2);
        }
        catch (InvalidKeyException ex) {
            return false;
        }
    }
    
    @Override
    public int hashCode() {
        try {
            byte[] encodedInternal = this.getEncodedInternal();
            int length = encodedInternal.length;
            for (int i = 0; i < encodedInternal.length; ++i) {
                length += (encodedInternal[i] & 0xFF) * 37;
            }
            return length;
        }
        catch (InvalidKeyException ex) {
            return 0;
        }
    }
    
    static void encode(DerOutputStream derOutputStream, AlgorithmId algorithmId, BitArray bitArray) throws IOException {
        DerOutputStream derOutputStream2 = new DerOutputStream();
        algorithmId.encode(derOutputStream2);
        derOutputStream2.putUnalignedBitString(bitArray);
        derOutputStream.write((byte)48, derOutputStream2);
    }
}
